package Presentator;
use strict;
use XML::SAX::ParserFactory;
use XML::SAX::Writer;

# actual implementation uses only SAX but DOM may be provided as well
use Presentator::PresentatorSAX;
use Carp;

use vars qw/$VERSION/;
$VERSION = '0.05';

sub new {
	my $proto = shift;
	my $class = ref $proto || $proto;
	
	my %attr = @_;
	
	my $output_string = '';
	my $writer = XML::SAX::Writer->new(Output=>\$output_string);
	my $handler = new Presentator::PresentatorSAX(%attr, Handler => $writer);
		
	my $factory = XML::SAX::ParserFactory->new(
		RequiredFeatures => {
			'http://xml.org/sax/features/namespaces' => 1,
		}
	);
	my $parser = $factory->parser( Handler => $handler);

	my $self  = {
		handler => $handler,
		parser => $parser,
		output => $handler,
		output_string => \$output_string,
	};
	
	bless $self, $class;
	return $self;
}

sub handler {
	return $_[0]->{handler};
}

sub parser {
	return $_[0]->{parser};
}

sub parse {
	my ($self, $fh) = @_;
	${$self->{output_string}} = '';
	
	croak("\$Presentator->parse should be used as \$pres->parse(\$IOHandle)!.")
		unless $fh;
	$self->parser->parse_file($fh);
	return Presentator::Output->new(string => $self->{output_string});
}

package Presentator::Output;

use XML::LibXML;

sub new {
	my ($proto, %attr) = @_;
	my $class = ref $proto || $proto;
	
	my $self = {
		string => (ref $attr{string})?($attr{string}):(\{$attr{string}}),
	};

	bless $self, $class;
}

sub getString {
	return ${$_[0]->{string}};
}

sub getStringRef {
	return $_[0]->{string};
}

sub getLibXML {
	my $parser = XML::LibXML->new();
	my $doc = $parser->parse_string($_[0]->getString);
	return $doc;
}

__END__
=head1 NAME

Presentator - XML processor for processing database presentations

=head1 SYNOPSIS

	use Presentator;
	use Presentator::QueriesFactory;
	use IO::File;

	my $qf = new Presentator::QueriesFactory();

	my $pres = Presentator->new(
		queriesFactory => $qf,
		URI => 'http://web.quick.cz/ichladil/presentator'
	);

	my $fh = new IO::File("myfile.xml");

	my $output = $pres->parse($fh); # parsing via XML::SAX
	$output->getString; # the output is string in UTF-8
	$output->getStringRef; # the output is string reference in UTF-8
	$output->getLibXML; # the output is XML::LibXML object

=head1 DESCRIPTION

This module is the entry point for the presentator. It is used to initiate parsing and to collect the results.

=head1 INTERFACE

=over 4

=item parse($IOhandle)

Parses XML document from C<$IOhandle> and returns the output object.

=back

=head1 TODO

=over 4

=item *

Add formatting possibilities based upon the field types.

=item *

Add debugging or exception possibilities.

=item *

Improve subqueries support.

=back

=head1 AUTHOR

Copyright 2002, Igor Chladil (http://web.quick.cz/ichladil/). This module may be used, copied and redistributed under the same terms as the Perl itself.

	
