package Presentator::QueriesFactory;

use strict;
use Carp;

=head1 NAME

Presentator::QueriesFactory - the factory class used to instantiate proper Queries and Select objects

=head1 SYNOPSIS

	use Presentator::QueriesFactory;

	my %all = Presentator::QueriesFactory->listQueries;
	while (my ($name, $attr) = each(%all)) {
		print "$name ... $attr->{documentation}";
	}
	
	my $qf = new Presentator::QueriesFactory(
		createQueriesFile => $file,
		dbh => $dbhandle,
	);
	
	my $queries = $qf->createQueries("MyData");
	my $select = $qf->createSelect("MyData");

=head1 DESCRIPTION

This module should provide means for Presentator extensions. It is the boundary class that hides all the informations needed for instantiation of Select and Queries objects. The parameters may be passed to the factory object. All the parameters are passed further to the particular Queries and Select objects.  

This is the module that should change whenever new types of Queries and Select objects are defined.

=cut

sub new {
	my ($proto, %attr) = @_;
	my $class = ref $proto || $proto;

	my $self = {
		atts => {%attr}, 
		QUERIES => {},
	};

	bless $self, $class;
}

sub createQueries {
	my ($self, $name) = @_;
	my $atts = $self->{atts};
	if (0) {
	} else {
		my $file = $atts->{createQueriesFile};
#		return $QUERIES{":$file"} if exists $QUERIES{":$file"};
		use Presentator::Queries;
		use IO::File;
		use XML::LibXML;
		my $xp_parser = XML::LibXML->new();
		my $xp_file = new IO::File("< $file");
		my $xp_doc = $xp_parser->parse_fh($xp_file);
		my $xp_root = $xp_doc->documentElement;
		my $queries = Presentator::Queries->new(xpath => $xp_root);
#		$QUERIES{":$file"} = $queries;
		return $queries;
	}
}

sub createSelect {
	my ($self, $name) = @_;
	my $atts = $self->{atts};
	if (0) {
	} else {
		use Presentator::Select;
		use Presentator::SecureParms;
		my $queries = $self->createQueries($name, $atts);
		croak "Queries object couldn't be instantiated." unless $queries;
		
		my $secparams = $atts->{secparams};
		$secparams = new Presentator::SecureParms(
			securityClasses => {
				"environment env" => 100,
				"template tmp" => 101,
				"default def" => 102,
				"_presaccess _pa" => 103,
			},
		) unless $secparams;
		return Presentator::Select->new(
			secparams=> $secparams, 
			queries => $queries,
			dbh => $atts->{dbh},
			);
	}
}

sub listQueries {
	return (
		"" => {documentation => "This are the default Queries and Select objects"}
	)	
}

1;

__END__

=head1 INTERFACE

=over 4

=item createQueries($name) 

C<$name> parameter identifies the required C<Queries> object. This method should return corresponding object, that should be created with set of attributes in mind. 

C<$name> is string.

=item createSelect($name)

dtto

=item listQueries

Returns the hash containing available C<Queries> and C<Select> objects. Names are the keys and values are hashrefs containing further description of the C<Queries> object. 

=back

=head1 AUTHOR

Copyright (c) 2002 Igor Chladil (ichladil@quick.cz). All rights reserved.

This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
