package Presentator::SecureParms;

use strict;
use Carp;
use vars qw ($VERSION);

BEGIN {
	$VERSION='0.01';
}

=head1 NAME

Presentator::SecureParms - implements secure parameters list 

=head1 SYNOPSIS

	#! /usr/bin/perl

	use strict;
	use Presentator::SecureParms 0.01;

	my $parms = Presentator::SecureParms->new(
		securityClasses => {
			"environment env" => 1,
			"external_file" => 2, 
			"user_defined" => 3
		},
		parms => {
			1 => {
				"userId" => 1455, 
				"faculty" => ["UHK"]},
			2 => {
				"userId" => 1400}
		},
		precedence => [1,2,3]
	);

	#returns 1455
	print $parms->get("userId"),"\n"; 
	
	$parms->setPrecedence([2,1,3]),"\n";
	
	#returns 1400
	print $parms->get("userId"),"\n"; 
	
	#return 1455, local precedence = [1]
	print $parms->get("userId", 2),"\n"; 
	$parms->set("userId", 1800, 3);
	
	 #returns 1455
	print $parms->get("userId", 3),"\n";
	
	#returns undef
	print $parms->get("userId", []),"\n"; 
	
	#returns 1800
	print $parms->get("userId", ["user_defined",2,1]),"\n"; 

=head1 DESCRIPTION

This package provides the class that implements secure parameter list.
It implements the list of variables in standard name=value format but it 
extends this concept by security levels. Each parameter has the security class
assigned. Security classes should be defined according to the problem solved, e.g. environement class, external file class, etc. Then the parameters may be extracted in certain security (e.g. environment parameters have precedence) order and with security constraints (e.g. only environment parameters).

=cut

#############################################################
#				Public Methods																		  #
#############################################################

=over 4

=item C<new({-securityClasses=E<gt>{},-parms=E<gt>{}});>

Returns a reference to an object that may be used to collect 
name=value pairs with regards to security level.

The parameters are optional. -securityClasses defines possible classes for 
security evaluation. Only the parameters of certain security classes may be
used at different moments of program processing. The number is assigned to each class and this is followed by tokens of synonymous names (aliases) for that class.

The parameter -parms defines the variables within certain security classes.
It is the Hash of Hash. The first level of hash defines the security class,
the second level defines name =E<gt> value pairs.

=cut

sub new {
	my $st = shift;
	my $class = ref $st || $st;
	my $self = bless {}, $class;
	my %args = @_;

	# This hash defines the reversed hash of security classes. The keys are the names and the values are assigne numbers. 
	foreach (keys %{$args{securityClasses}}) {
		my $v = $args{securityClasses}{$_};
		$self->{securityClasses}{$v} = $v;
		foreach my $alias (split /\s+/, $_) {
			$self->{securityClasses}{$alias} = $v;
		}
	}
	
	$self->{parms} = 
		($args{parms} or {});

	# the default precedence is set in ascending order of security defs.
		if (defined $args{precedence}) {
			$self->setPrecedence($args{precedence});
		} else {
			my @precedence = sort {$a <=> $b} values(%{$self->{securityClasses}});
			$self->setPrecedence(\@precedence);			
		}
		
	return $self;
}

=item C<getSecurityClasses()>

Returns the hash of current defined security classes. The form is the same
as for the C<new> subroutine.

=cut

sub getSecurityClasses {
	my $self = shift;
	return %$self->{securityClasses};
}

=item C<setSecurityClasses($name,$order)>

(re)Defines the name and order of certain security class.

=cut

sub setSecurityClass {
	my ($self, $scName, $scOrder) = @_;
	$self->{securityClasses}{$scName} = $scOrder;
}

=item C<getPrecedence()>

Returns current set of precedence array. 

=cut

sub getPrecedence {
	my $self = shift;
	return @{$self->{precedence}};
}

=item C<setPrecedence($array_ref)>

Sets the precedence order that will be taken into account when looking for parameter value. The value will be taken from the first security class in precedence order.

=cut

sub setPrecedence {
	my $self=shift;
	$self->{precedence} = $_[0];
}

=item C<getParameters([$sec_class])>

Returns the HoH with current defined parameters in the same format as in the C<new> subroutine. If the security class is defined than only the parameters of that class are returned.

=cut

sub getParameters {
	my $self = shift;
	return %{$self->{parms}} if not $_[1];
	return %{$self->{parms}{$self->{securityClasses}{$_[1]}}};
}

=item C<get($name,$maxSecurityLevel|$precedence_ref)>

Returns the value of the parameter named C<name>. The precedence is accounted for. maxSecurityLevel defines the maximum level of insecurity possible for the parameter. It may be substituted by reference to the list of precedence, where the first value in defined precedence is returned.

=cut


sub get {
	my $self = shift;
	my $name = shift;
	my @precedence = @{$self->{precedence}};
	if ($#_ != -1) {
		if (!ref $_[0]) { # If the parameter is scalar, then it is interpreted as the maximum security level
			@precedence = ();
			foreach $_ (values %{$self->{securityClasses}}) { 
				push(@precedence, $_) unless $self->{securityClasses}{$_[0]}<$_;
			}
			@precedence = sort {$a <=> $b} @precedence;
		}
		# if the parameter is ArrayRef then it is taken for precedence array
		if (ref $_[0] eq 'ARRAY') {
			@precedence = map($self->{securityClasses}{$_},@{$_[0]});
		}
	}
	#now the value is searched in precedence order
	foreach (@precedence) {
		return $self->{parms}{$_}{$name} 
			if $self->{parms}{$_}{$name};
	}

	#or undef is returned
	return;
}

=item C<set($name,$value,$securityClass)>

Sets the name=value pair in parameters of the given securityClass. 

=cut

sub set {
	my ($self,$name,$value,$sc) = @_;
	$self->{parms}{$self->{securityClasses}{$sc}}{$name} = $value;
}

=item C<delete($securityClass [,$name])>

Deletes all the parameters from the given security class or only certain parameter from the given security class.

=cut

sub delete {
	my ($self, $sc, @params) = @_;
	$sc = $self->{securityClasses}{$sc};
 	if (!@params) {
		delete $self->{parms}{$sc}
	} else {
		map(
			delete $self->{parms}{$sc}{$_}, 
			@params)
	}
}

=back

=head1 AUTHOR

Copyright (c) 2002 Igor Chladil (ichladil@quick.cz). All rights reserved.

This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.

=cut

1;
