############################################################
package Presentator::Select;
############################################################
use strict;
use vars qw/$VERSION/;
use Presentator::SQLResults;
use Presentator::SQLField;
use Carp;

$VERSION = "0.050";

1;

=head1 NAME

Presentator::Select - data retriever

=head1 SYNOPSIS

	#! /usr/bin/perl -w

	use strict;
	use Presentator::Queries;
	use Presentator::Select;
	use XML::LibXML;
	use DBI;

	# DB Handle is created
	my $dbh = DBI->connect(...); 

	# Queries object is created
	my $xparser = XML::LibXML->new();
	$xp = $xparser->parse_fh(new IO::File("< data.xml"));
	my $queries = new Presentator::Queries(
		xpath => $xp->documentElement);

	# Select object is created, no security is passed in
	my $select = new Presentator::Select(
		queries => $queries,
		secparams=> undef);

	# The interface usage is examplified

	# the requirements for the output are gathered
	$select->addFieldTag("id");
	$select->addFieldTag("header");

	# the explicit conditions for the output are gathered
	$select->addWhereTag("keywords", keyword => "ISDN");

	# the SQLResults are returned
	my $results = $select->select([]);

=head1 DESCRIPTION

This class performs data-retrieval. Retrieved data are identified by a few inputs. It gathers the requirements for the information (fields, outputs), the conditions these information should met in order to be valid (where tags), the order in which they appear in the results (order tags).  

Following stands for this particular class. All these requirements are transformed into SQL query. The transformation is supported by the Queries object that has the information necessary for gluing the requirements into the SQL query. Query is then performed and retrieved data are returned as C<Presentator::SQLResults> object.

=cut

sub new {
	my ($class, %attr) = @_;
	$class = ref $class || $class;
	my $self = {
		queries => $attr{"queries"}, # Queries object
		secparams => $attr{"secparams"}, # The SecureParams object 
		fields => {}, # Requirements for the output
		conditions => [], # Conditions for the output, only the explicit ones
		_conditionStack => [], # the stack for logical conjunctions

		subqueries_cnd => [], # defines the subquery objects (this object is in the role of subquery
		subquery_parms => {}, # This is the hash of index references into the binding parameters array, only the params with class _subqueryparm are here

		order_by => [], # name of the fields according to which I should order
		
		binds => [], # Parameters for the binding
		binds_fields => [], # The same as above, but for field parameters (security purposes)
		
		warnings => '', # The buffer for the warnings
		debug_buf => '', # The buffer for the debugging messages.

		bindingNames => { 
			template => "template", 
			default => "default",
			environment => "env"},

		access_groups=> undef,

		error => undef, # actual state of error
		
		debug => 0,
	};
	bless $self, $class;
	my $groups = $self->secparams->get("groups",["_presaccess"]);
	my @groups = split(/\s+/, $groups) if $groups;
	$self->{access_groups} = \@groups if $groups;
	if (exists $attr{debugCallback}) {
		$self->setDebugCallback($attr{debugCallback});
	} else {
		$self->setDebugCallback(sub { warn("$_[1]:\t$_[0]\n"); });
	}
	$self->{dbh} = $attr{dbh} or $self->warn("There is no DB Handle passed to the Presentator::Select");
	return $self;
}

sub setDebugCallback {
	my ($self, $callback) = @_;
	croak('Expected usage: setDebugCallback(\&callback)') unless $callback;
	$self->{debugCallback} = $callback;
}

sub debug {
	my $self= shift;
	my $callback = $self->{debugCallback};
	my $msg = shift;
	my $level = shift || 1;
	&$callback($msg, "debug") if $self->{debug}>=$level;
}

#The warn method adds the string into the warning buffer.
sub warn {
	my ($self, $msg) = @_;
	my $callback = $self->{debugCallback};
	&$callback($msg, "warning");
}

sub error {
	my $error = $_[0]->{error};
	$_[0]->{error} = $_[1];
	return $error;
}

sub queries {
	my $queries = $_[0]->{queries};
	$_[0]->error;
	if (!$queries->isAllowed($_[0]->{access_groups})) {
		$_[0]->warn("You have not the access rights to the queries object");
		$_[0]->error(1);
	} else {
		return $queries;
	}
	return undef;
}

sub query {
	my $self=$_[0];
	$self->error;
	my $queries = $_[0]->queries();
	if ($queries and $_[0]->getQueryName()) {
		my $query = $queries->query($_[0]->getQueryName());
		if (!$query) {
			$self->warn("There is no query named ".$_[0]->getQueryName());
			$_[0]->error(2);
		} elsif (!$query->isAllowed($_[0]->{access_groups})) {
			
			$self->warn("You have not the access rights to the query ".
				$_[0]->getQueryName());
			$_[0]->error(1);
		} else {
			return $query;
		}
	}
	return undef;
}


sub field {
	my $self=$_[0];
	$self->error;
	my $query = $_[0]->query();
	if ($query) {
		my $field=$query->field($_[1]);
		if (!$field) {
			$self->warn("There is no field named $_[1]");
			$self->error(2);
		} elsif (!$field->isAllowed($_[0]->{access_groups})) {
			$self->warn("You have not the access rights to the field $_[1]");
			$self->error(1);
		} else {
			return $field;
		}
	}
	return undef;
}


sub condition {
	my $self=$_[0];
	$self->error;
	my $query = $_[0]->query();
	if ($query) {
		my $f=$query->condition($_[1]);
		if (!$f) {
			$self->error(2);
			$self->warn("There is no condition named $_[1]");
		} elsif (!$f->isAllowed($_[0]->{access_groups})) {
			$self->error(1);
			$self->warn("You have not the access rights to the condition $_[1]");
		} else {
			return $f;
		}
	}
	return undef;
}

sub secparams {
	return $_[0]->{secparams};	
}

sub setQueryName {
	$_[0]->{"query_name"} = $_[1];
}		

sub getQueryName {
	return $_[0]->{"query_name"};
}

# addOrderBy({name=>$field,order=>$type})
sub addOrderBy {
	my $self = shift;
	my $attr = shift;
	# I should add this field into the required fields array at first even though I won't use its value 
	$self->addFieldTags($attr->{"name"});
	$attr->{"type"} = ($attr->{"type"} or "");
	push(@{$self->{order_by}}, $attr->{"name"}." ".$attr->{"type"});
	$self->debug("Order by added: ".join(',',@{$self->{order_by}}),3);
}

#addFieldTags(@tag_names)
sub addFieldTags {
	my ($self, @tags) = @_;
	map { 
			$self->{fields}{$_} = scalar(keys %{$self->{fields}}) 
				unless exists $self->{fields}{$_};
		} @tags;
}

## This function is internal to this package and serves for binding parameters
sub _bindParameters {
	my ($self, $cnd, $binding_classes_def, $binds_array) = @_;
	foreach ($cnd->params()) {
		# check whether the parameter class is not the _subqueryparam
		if ($_->classes() and ($_->classes->[0] eq '_subqueryparam')) {
			push @{$self->{binds}}, $_->name;
			$self->{subquery_parms}{scalar(@{$self->{binds}})-1} = $_->{name};
			next;
		}
		
		# default classes order may be overriden
		my $binding_classes = $binding_classes_def;
		$binding_classes = $_->classes() if defined $_->classes();

		$binds_array = $self->{binds} if not $binds_array;
		
		#set the default parameter value
		$self->secparams->set($_->name, $_->default, 
			$self->{bindingNames}{default});
		# bind the parameter
		push @{$binds_array}, 
			$self->secparams->get($_->name, $binding_classes);

		# unset the default parameter value
		$self->secparams->delete($self->{bindingNames}{default});
	}
}

#addWhereTag($tag,%attr)
sub addWhereTag {
	my ($self, $tag, %attr) = @_;
	$self->warn("No query is selected"),return unless $self->query();
	# if this is the and|or group start/end then add the parenthesis into
	# the conditions array
	if ($tag eq 'not') {
		push(@{$self->{conditions}}, $self->{_conditionStack}[-1])
			if ($self->{conditions}[-1] !~ /^(\(|and|or)$/);
		push @{$self->{conditions}}, ' not ';
		push @{$self->{conditions}}, '(';
	} elsif ($tag eq 'not_end') {
		push @{$self->{conditions}}, ')';
	} elsif ($tag=~ /^(and|or)$/io) {
		push @{$self->{_conditionStack}}, lc($tag);
		push @{$self->{conditions}}, '(';
	} elsif ($tag =~ /^(and_end|or_end)$/io) {
		# testing whether the array was not empty
#		if (($self->{conditions}[-1] eq '(') and (not ref $self->{conditions}[-1])) {
#			pop @{$self->{conditions}};
#		} else {
		push @{$self->{conditions}}, ')';
#		}
		pop @{$self->{_conditionStack}};
	} elsif (($tag =~ /^(gt|lt|eq|ge|le|like|in)$/o) and 
			$self->query()->field($attr{"name"})) {
			my $cnd = $attr{"name"};
			$self->addFieldTags($cnd);
			if ($tag =~ /^(gt|lt|eq|ge|le|like)$/o) {
				my $tr = $tag;
				$tr =~ s/gt/>/; 
				$tr =~ s/lt/</;
				$tr =~ s/eq/=/; 
				$tr =~ s/ge/>=/; 
				$tr =~ s/le/<=/;
				$cnd .= " $tr ?";
				push @{$self->{binds}}, $attr{"value"};
			} elsif ($tag eq 'in') {
				$cnd .= " $tag ($attr{value})";
			}
			push(@{$self->{conditions}}, $self->{_conditionStack}[-1])
				if ($self->{conditions}[-1] !~ /^(\(|and|or)$/);
			push @{$self->{conditions}}, " ( $cnd ) ";
	} elsif (!$self->query()->condition($tag)) {
		$self->warn("There is no condition named '$tag' or it isn't accessible.")
	} else {
		my $cnd = $self->query()->condition($tag);
		# add the logical conjunction when it is necessary
		push(@{$self->{conditions}}, $self->{_conditionStack}[-1])
			if ($self->{conditions}[-1] !~ /^(\(|and|or)$/);
			
		# add the placeholding string on the conditions stack
		push @{$self->{conditions}}, $cnd;

		# Add all binding params into secparams with security class = template
		foreach (keys %attr) {
			$self->secparams->set($_, $attr{$_}, $self->{bindingNames}{template});
		}
		# add binding parameters on the binding stack. I do it via security classes that takes care of right binding. The names of internal binding parameters are defined in the hash bindingNames
		$self->_bindParameters($cnd, 
			[$self->{bindingNames}{template}, $self->{bindingNames}{default}]);
		
		# no junk should be left
		$self->secparams->delete($self->{bindingNames}{template});
	}
}

#addHasSubquery($subquery_name,$role)
#(the subqueries are usually bound by extra conditions and these conditions have the parameters from the parent query).
sub addHasSubquery {
	my ($self, $name, $as) = @_;

	$self->warn("No role was specified within subquery.") unless $as;

	# At first I should find the right subquery object
	foreach my $sq ($self->query->hasSubquery()) {
		
		if (($sq->{role} eq $as) and 
			($sq->{subquery}->name eq $name) and
			($sq->{sources}[0]->name eq $self->getQueryName()) ) {
			$sq->{condition}->addwhere();
			foreach my $aq ($sq->{condition}->addwhere()) {
				map {
				$self->addFieldTags($_->name) 
					if $_->classes->[0] eq "_subqueryparam" } $aq->params();
			}
		}
	}
	$self->debug("Added subquery $name as $as",3);
}

#addIsSubquery($parent_query_name, $role)
#(e.g. add the respective subquery conditions and inform about the parameters binding). 

#The subquery parameters should be bind before actual select, that is the great difference against where and addwhere tags, that makes the parameters binding immediately (because these parameters are fixed). 

sub addIsSubquery {
	my ($self, $par_name, $as) = @_;
	# first I should find the subquery object

	foreach my $sq ($self->query->isSubquery()) {
		if (($sq->{role} eq $as) and 
			($sq->{sources}[0]->name eq $par_name->[0]) ) {
				push @{$self->{subqueries_cnd}}, $sq->{condition};
				$self->debug("Added 'is' subquery from $par_name->[0] as $as",3);
				return;
		}
	}
	$self->warn("There is no such subquery $par_name->[0] in the role $as in ".$self->getQueryName());
}

#=item getSelect()
#
#Returns SQL select generated on the basis of provided values. The SQL contains the placeholders, into which the respective values should be bound.
#

sub getSelect {
	my $self = shift;
	return unless $self->{"fields"};
	my $str = "";
	my $query = $self->query();
	return unless $query;
	$self->addFieldTags(@{$query->group_by}) if $query->group_by;
	
	$str = "SELECT distinct ";
	$str .= 
		join(", ",
			map {
				my $pom = $self->field($_); 
				if ($pom) {
					$self->_bindParameters($pom, 
						[$self->{bindingNames}{environment}], $self->{binds_fields});
					$pom->sql()." as $_";
				} else {
					"'xxxxxx' as $_";
				}
			}
			sort {$self->{fields}{$a} <=> $self->{fields}{$b} }
				keys %{$self->{fields}}
		);

	# Create FROM part of the documents
	my @implcnds = (
			$query, #query part 
			@{$self->{subqueries_cnd}} # subqueries parts
		);
	map {push(@implcnds, $self->field($_))
		if $self->field($_)} keys %{$self->{fields}}; # fields part
	map {push @implcnds, $_ if ref $_} @{$self->{conditions}}; # explicit where implicit parts
	my %froms = ();
	foreach my $implcnds (@implcnds) {
		map { $froms{$_}=1 if $_} $implcnds->addfrom()
	}
	if (%froms) {
		$str .= "\nFROM ";
		$str .= join(", ",keys %froms);
	}
	
	my @where = ();
	# process explicit where, these must be first because their parameters are bound first
	push (@where,  "(".
		join(' ',map (
			((ref $_)?
			("(".$_->placeholdingString().")"):
			($_)), @{$self->{conditions}} ))
		.")" ) if @{$self->{conditions}};
	# Create where part of the document, the implicit wheres
	my %wheres = ();
	foreach my $implcnds (@implcnds) {
		map {	$wheres{$_}=$_ } $implcnds->addwhere();
	}

	# process implicit where, including securities 
	push (@where, "(".join(" AND ",
		map { 
			$self->_bindParameters($_, 
				[$self->{bindingNames}{environment},$self->{bindingNames}{default}]);
			$_->placeholdingString();
			} values %wheres).")") 
		if %wheres;
	
	$str .= "\nWHERE ".join(" AND ", @where) if @where;
	
	if (@{$self->{order_by}}) {
		$str .= "\nORDER BY ".join(', ', @{$self->{order_by}})
	}
	
	if ($query->group_by) {
		$str .= "\nGROUP BY ".join(', ', @{$query->group_by})
	}
	
	return $str;
}

#select($pre_selects,$pre_results,{dbh=E<gt>$dbh})
sub select {
	my ($self, $pre_results) = @_;

	my $dbh = $self->{dbh};

	# caching SQL form of the select
	$self->{sql_select} = $self->getSelect() unless $self->{sql_select};
	# rebind the subquery params

	while (my ($index,$field) = each %{$self->{subquery_parms}}) {
		$self->{binds}[$index] = 
			$pre_results->[-1]->field($field)->value;
	}
	
	# query the database

	my $sth = $dbh->prepare($self->{sql_select}) or 
		$self->warn("Creating statement object failed with Select: $self->{sql_select}");
	$sth->execute(@{$self->{binds_fields}}, @{$self->{binds}}) if $sth;

	# create and return the query results iterator
	my %fields;
	foreach (keys %{$self->{fields}}) {
		my $qf = $self->query->field($_);
		my $field = Presentator::SQLField->new(
			name => $qf->name(),
			type => $qf->type(),
			encoding => $qf->encoding(),
			"index" => $self->{fields}{$_},
		);
			
		$fields{$_} = $field;
	}
	
	return Presentator::SQLResults->new(
		fields=>\%fields,
		sth   =>$sth,
		);
}
__END__

=head1 Methods specific to this implementation

=over 4

=item new(queries=E<gt>$queries,secparams=E<gt>$secparams)

The new Select object is created. Queries object is the required input. 

=item debug($msg,$level=1)

Prints the debugging message into the buffer according to the debugging level.

=item error($error_code) 

This method returns the previous error code and sets current error code to the $error_code.

=item queries()

Returns the Queries object.

=item query()

Returns the current Presentator::Query object or undef if it is not defined.

=item secparams()

Return the SecureParms object.

=item getQueryName()

Returns the name of the current query.

=back

=head1 INTERFACE

Following paragraphs describes the interface of Select objects. Every implementation of Select should fulfill following syntax and semantics.

=over 4

=item setQueryName($new_name)

Set the name of the query to the $new_name. The current query definition may be obtained via query().

=item addOrderBy({name=E<gt>$field,order=E<gt>$type})

This method adds the order by clause into the select. The parameter into this function is the hash reference containing the name of the output field by which the output should be ordered and the type of the order, i.e. ascending (ASC) x descending (DESC).

=item addFieldTags(\@tagNames)

This function is used for adding the requirements for the information. The required information is divided into the atomic fields. The requirement is usually fired by the respective XML elements in the template (iterate content). 


Particular implementation details - not part of the interface: The name of the tags references the respective field in the Queries definition object. This method only takes the evidence of data requirements; it makes no further processing.

=item addWhereTag($tag,%attr)

The method addWhereTag adds the requirement for the explicitly (in the template) defined constraints. The C<$tag> is the tag name (element starting tag) from the XML template. Attributes C<%attr> is the hash of parameter names with associated parameter values.

The following tags require special treatment: C<and>, C<or>, C<not>, C<and_end>, C<or_end>, C<not_end>. These tags identifies start/end tags of logical operators elements.

B<Following list is not the part of interface. It is the part of this particular implementation!!!>
The constraints in this implementation are of three types.

=over 4

=item 1.

The constraints that are not constraints at all. These are the C<and> and C<or> tags that serves for grouping by logical conjunctions. The conditions within each group are joined by respective logical conjunction.

=item 2.

The constraints that are explicitly defined in Queries. The %attr hash holds the binding parameters for the constraint.

=item 3.

The built-in (default) constraints that serves for simple constraining of fields. These are the simple conditions like C<gt> (greater than), C<lt> (less than), C<ge> (not less than), C<le> (not greater than), C<eq> (equal), C<like> (like), C<in> (in).

=back

=item addHasSubquery($subqueryName,$role)

The method addHasSubquery informs the query that it has the subquery and the select object may take respective actions. C<$subqueryName> identifies the query that acts as the subquery and the C<$role> identifies the relation of the subquery to this query. 

=item addIsSubquery($parentQueryName, $role)

The method addIsSubquery informs the subquery about the fact that it is subqueryand that it should take respective actions to act as the subquery. C<$parentQueryName> informs about query that acts as the parent query. The C<$role> informs subquery about the relation to the parent query.

=item select($pre_sqlresults)

This function performs actual select. Previous SQLResults C<$pre_sqlresults> is used for binding parameters in the subquery. 

It returns C<Presentator::SQLResults> object or undef.

=item new(debugCallback =E<gt> \&callback)

Each C<Select> implementation should allow debugging information to be passed to the callback function. Parameters to the callback function are C<message> and the C<source>. Source may be C<debug>, C<warn> or C<error>.

Callback function is passed through the constructor C<new>, which is called from C<QueriesFactory>.

=back

=head1 AUTHOR

Copyright (c) 2002 Igor Chladil (ichladil@quick.cz). All rights reserved.

This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.


